# Prepare package
library(survival)
library(qvalue)
library(stringr)

# Data loading
# MESO RNA-seq dataset and MESO RNA-seq clinical dataset from cBioPortal (https://www.cbioportal.org/)
expression_raw = read.csv("MESO RNA-seq.csv") 
outcome_raw = read.csv("MESO RNA-seq clinical.csv",skip=4,header=TRUE)

# Preprocessing clinical data
outcome_OS_raw = outcome_raw[,c(1,31,32,5,6,7,19,26)]
outcome_OS = outcome_OS_raw
outcome_OS[,"OS_STATUS"] = as.numeric(substr(outcome_OS_raw[,"OS_STATUS"],1,1))
outcome_OS[,"SEX"] = as.factor(outcome_OS_raw[,"SEX"])
outcome_OS[,"STAGE"] = as.factor(ifelse(as.character(outcome_OS_raw[,"AJCC_PATHOLOGIC_TUMOR_STAGE"]) %in% c("STAGE IA","STAGE IB"), "STAGE I",as.character(outcome_OS_raw[,"AJCC_PATHOLOGIC_TUMOR_STAGE"])))


# Excluding subjects with missing data in OS from clinical dataset
exc_row = c(which(outcome_OS[,"OS_MONTHS"]==0),which(is.na(outcome_OS[,"OS_MONTHS"])))
outcome_OS_comp = outcome_OS[-exc_row,]

geneid_symbol = paste(expression_raw[,2],expression_raw[,1],rownames(expression_raw),sep="_")
expression_raw1 = cbind(geneid_symbol,expression_raw[,-c(1,2)])

# Excluding subjects with missing data in OS from RNA dataset
naid = outcome_OS[is.na(outcome_OS[,"OS_MONTHS"]),1]
zeroid = outcome_OS[which(outcome_OS[,"OS_MONTHS"]==0),1]
exc_id = c(zeroid,naid)
nacolnum = which(naid==gsub("\\.","-",substr(colnames(expression_raw1),1,12)))
zerocolnum = which(zeroid==gsub("\\.","-",substr(colnames(expression_raw1),1,12)))
expression_t = expression_raw1[,-c(1,zerocolnum,nacolnum)]
geneid = expression_raw1[,1]

# Scaling RNA expression
expression = t(expression_t)
colnames(expression)=geneid

expression_scale = apply(expression,2,scale)


# Genes with zero expression in more than 10 samples are excluded.
ftemp = function(x){sum(x==0)}
expression_nzero = apply(expression,2,ftemp)
inc = expression_nzero<10
expression_scale_inc = expression_scale[,inc]

# Fitting multivariate Cox model adjusted by age, sex and stage
coefficients_mat = NULL
for(i in 1:ncol(expression_scale_inc)){
    exp = expression_scale_inc[,i]
    df_adj = cbind(outcome_OS_comp[,c("OS_STATUS","OS_MONTHS")],exp,outcome_OS_comp[,c("AGE","SEX","STAGE")])
    cox_adj_i = coxph(Surv(OS_MONTHS,OS_STATUS)~exp+AGE+SEX+STAGE,data=df_adj)
    coef = c(summary(cox_adj_i)$coefficients[1,], summary(cox_adj_i)$conf.int[1,])
    coefficients_mat = rbind(coefficients_mat,coef)
}

# Multiple testing base on FDR (Storey's method)
pvec = coefficients_mat[,5]
qv_res = qvalue(pvec,fdr.level = 0.01)
plot(qv_res)
table(qv_res$significant)

# Make dataset for output sorted by p-value
rownames(coefficients_mat) = colnames(expression_scale_inc)
adj_res = coefficients_mat[qv_res$significant,]
sortind = order(adj_res[,5])
adj_res_sort = adj_res[sortind,]
temp_adj = str_split(rownames(adj_res_sort),"_",simplify = TRUE)
gene_symbol_sort_adj = temp_adj[,2]
gene_id_sort_adj = temp_adj[,1]
colnames(adj_res_sort) = c(colnames(summary(cox_adj_i)$coefficients), 
                                colnames(summary(cox_adj_i)$conf.int))

detected_gene = as.data.frame(cbind(gene_symbol_sort_adj,gene_id_sort_adj,adj_res_sort[,c(2,8,9,5)]))

# write.csv(detected_gene,"detected_gene.csv",row.names=FALSE)


